package com.persagy.data_dict_tool;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.pbsage.util.Constant;
import com.pbsage.util.HttpUtil;
import com.persagy.util.DataDictUtil;
import com.persagy.util.ExcelSheet;
import com.persagy.util.ExcelUtil;

public class DataDictScanner {
	public static void main(String[] args) throws Exception {
		Scan_object();
	}

	public static void Scan_object() throws Exception {
		JSONObject classTree = JSON.parseObject(HttpUtil.Instance().get(DataDictToolMain.url + "dict/class"));
		// Constant.warn(JSON.toJSONString(classTree, SerializerFeature.WriteMapNullValue));
		// Constant.warn(FastJsonUtil.toFormatString(classTree));
		JSONArray dataArray = (JSONArray) classTree.get("data");

		List<String> titleColList = new ArrayList<String>();
		titleColList.add("sign");
		titleColList.add("funcid");
		titleColList.add("dataType");
		ExcelSheet report = new ExcelSheet();
		report.titleColList = titleColList;
		ExcelSheet set = new ExcelSheet();
		set.titleColList = titleColList;
		Scan_object_Array(dataArray, report, set);

		Map<String, ExcelSheet> reportMap = new HashMap<String, ExcelSheet>();
		reportMap.put(DataDictToolMain.projectId, report);
		ExcelUtil.Write("report.xlsx", reportMap);

		Map<String, ExcelSheet> SetMap = new HashMap<String, ExcelSheet>();
		SetMap.put(DataDictToolMain.projectId, set);
		ExcelUtil.Write("set.xlsx", SetMap);
	}

	public static void Scan_object_Array(JSONArray dataArray, ExcelSheet report, ExcelSheet set) throws Exception {
		for (int i = 0; i < dataArray.size(); i++) {
			JSONObject data = (JSONObject) dataArray.get(i);
			Scan_object_Object(data, report, set);
		}
	}

	private static void Scan_object_Object(JSONObject data, ExcelSheet report, ExcelSheet set) throws Exception {
		String classCode = (String) data.get("code");
		if (!classCode.startsWith("system") && !classCode.startsWith("equipment")) {
			Constant.warn(classCode);
			JSONArray infos = DataDictQueryUtil.get_infos(classCode);
			JSONArray objects = DataDictQueryUtil.getObjects(classCode);
			if (objects != null) {
				for (int index_o = 0; index_o < objects.size(); index_o++) {
					JSONObject object = (JSONObject) objects.get(index_o);
					String object_name = (String) object.get("localName");
					Constant.warn(object_name);
					for (int i = 0; i < infos.size(); i++) {
						JSONObject info = (JSONObject) infos.get(i);
						// Constant.warn(JSON.toJSONString(info, SerializerFeature.WriteMapNullValue));
						String code = (String) info.get("code");
						String dataType = (String) info.get("dataType");
						Object infoValue = object.get(code);
						if (infoValue == null || !(infoValue instanceof String)) {
							continue;
						}
						String infoValueString = (String) infoValue;

						List<String> columnList = new ArrayList<String>();
						int index_ = infoValueString.lastIndexOf('-');
						if (index_ != -1) {
							String sign = infoValueString.substring(0, index_);
							String funcid = infoValueString.substring(index_ + 1);
							columnList.add(sign);
							columnList.add(funcid);
						}
						columnList.add(dataType);
						if (DataDictUtil.getInfoType(info) == 1) {
							report.contentListList.add(columnList);
						} else if (DataDictUtil.getInfoType(info) == 2) {
							set.contentListList.add(columnList);

						}
					}
				}
			}
		}

		JSONArray children = (JSONArray) data.get("children");
		if (children != null) {
			Scan_object_Array(children, report, set);
		}
	}

	public static void Scan_infos() throws Exception {
		JSONObject classTree = JSON.parseObject(HttpUtil.Instance().get(DataDictToolMain.url + "dict/class"));
		// Constant.warn(JSON.toJSONString(classTree, SerializerFeature.WriteMapNullValue));
		// Constant.warn(FastJsonUtil.toFormatString(classTree));
		JSONArray dataArray = (JSONArray) classTree.get("data");
		Map<String, List<String>> map = new HashMap<String, List<String>>();
		Scan_infos_Array(dataArray, map);

		String[] keys = map.keySet().toArray(new String[0]);
		Arrays.sort(keys);
		for (String key : keys) {
			List<String> codeList = map.get(key);
			Constant.warn(key + "\t" + codeList.toString());
		}
	}

	public static void Scan_infos_Array(JSONArray dataArray, Map<String, List<String>> map) throws Exception {
		for (int i = 0; i < dataArray.size(); i++) {
			JSONObject data = (JSONObject) dataArray.get(i);
			Scan_infos_Object(data, map);
		}
	}

	private static void Scan_infos_Object(JSONObject data, Map<String, List<String>> map) throws Exception {
		String code = (String) data.get("code");
		if (!code.startsWith("system-") && !code.startsWith("equipment-")) {
			JSONArray infos = DataDictQueryUtil.get_infos(code);

			for (int i = 0; i < infos.size(); i++) {
				JSONObject info = (JSONObject) infos.get(i);
				// Constant.warn(JSON.toJSONString(info, SerializerFeature.WriteMapNullValue));
				String firstTag = (String) info.get("firstTag");
				String category = (String) info.get("category");
				String key = category + "-" + firstTag;
				if (!map.containsKey(key)) {
					map.put(key, new ArrayList<String>());
				}
				boolean exist = false;
				for (String codeInner : map.get(key)) {
					if (codeInner.equals(code)) {
						exist = true;
						break;
					}
				}
				if (!exist) {
					map.get(key).add(code);
				}
			}
		}

		JSONArray children = (JSONArray) data.get("children");
		if (children != null) {
			Scan_infos_Array(children, map);
		}
	}
}
